import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.*;

/**
 * Canvas for drawing the demo on.
 */
public class DemoCanvas extends GameCanvas implements Runnable {

    // Minimum milliseconds per frame.
    // 1000/fps should work, but it's not really exact.
    final static int MIN_MSPF = 1000/30;

    int FPS_frames = 0;     // frames accumulator
    int FPS_time = 0;       // time elapsed
    int FPS_fps = 0;        // most recent FPS calculation

    boolean skip = false;   // User wants to skip this part

    // XXX handle this
    VirtualReality vr;

    /**
     * Class constructor.
     */
    public DemoCanvas()
    {
        super(true);
    }

    /**
     * Initialize this canvas and start the main loop.
     */
    public void start()
    {
        // init
        // start
        Thread runner = new Thread(this);
        runner.start();
    }

    /**
     * Main loop for the running demo.
     */
    public void run()
    {
        boolean done = false;
        Graphics gfx = getGraphics();
        long lastTime = System.currentTimeMillis();

        // XXX handle this somehow
        vr = new VirtualReality(gfx, 0, 0, getWidth(), getHeight());
        vr.load();

        while(!done) {

            // Check user input
            checkInput();

            // Calculate time elapsed
            long now = System.currentTimeMillis();
            int deltaT = (int)(now - lastTime);
            lastTime = now;

            // XXX test the demo
            Actor cube = (Actor)vr.actors.elementAt(0);
//            cube.rotX += FP.multiply(0x000007AE, FP.whole(deltaT));
            cube.rotY += FP.multiply(0x000007AE, FP.whole(deltaT));
//            cube.rotZ += FP.multiply(0x000007AE, FP.whole(deltaT));
            /*
            cube.position[0] +=
                FP.multiply(0x00000040, FP.whole(deltaT));
            cube.position[1] =
                FP.multiply(0x00050000,
                        FP.sin(FP.multiply(0x00200000, cube.position[0])));
            */

            // Update screen
            updateScreen(gfx, deltaT);

            // Calculate FPS
            calcFPS(deltaT);

            // Sleep to cap the FPS
            try {
                if(deltaT < MIN_MSPF) {
                    int delay = MIN_MSPF - deltaT;
                    Thread.currentThread().sleep(delay);
                }
            } catch(Exception e) {}
        }
    }

    /**
     * Get user input, and set flags appropriately.
     */
    private void checkInput()
    {
        int keyState = getKeyStates();

        // Fire button: skip this part
        if((keyState & FIRE_PRESSED) != 0)
            skip = true;
    }

    /**
     * Paint the screen.
     */
    private void updateScreen(Graphics g, int deltaT)
    {
        g.setColor(0xffffff);
        g.fillRect(0, 0, getWidth(), getHeight());
        /*
        g.setColor(0x000000);
        g.drawString("W: "+getWidth()+
                 ",H: "+getHeight(),
                 1, 1, Graphics.TOP | Graphics.LEFT);
        g.drawString("dT: "+deltaT+
                 ",t: "+FPS_time+
                 ",FPS: "+FPS_fps,
                 1, 19, Graphics.TOP | Graphics.LEFT);
        */
        vr.paint();
        g.setColor(0x000000);
        g.drawString(FPS_fps+" fps", 1, 1, Graphics.TOP | Graphics.LEFT);

        flushGraphics();
    }

    /**
     * Do calculations related to how many frames/second we've blitted.
     * Call once after every frame drawn.
     */
    private void calcFPS(int deltaT)
    {
        FPS_frames++;       // increment frames drawn
        FPS_time += deltaT; // increase time taken

        // Update FPS calculation every 1000ms
        if(FPS_time >= 1000) {
            FPS_fps = FPS_frames * 1000 / FPS_time;
            FPS_time = 0;
            FPS_frames = 0;
        }
    }
}

/* vim: set ts=4 sw=4 sta sts=4 et ai: */
