#!/usr/bin/perl
#
# Check args
if($#ARGV != 1) {
	print "USAGE: mtoobj.pl <input.m> <output.obj>\n";
	exit 0;
}

# Prepare files
my ($infile, $outfile);
if(!open($infile, '<', $ARGV[0])) {
	print "Can't open M file ".$ARGV[0]."\n";
	exit 1;
}
if(!open($outfile, '>', $ARGV[1])) {
	print "Can't create OBJ file ".$ARGV[1]."\n";
	exit 2;
}

# Keep count of how many things we pack
my $vertices = 0;
my $faces = 0;

# Loop over every line in the input file
while(my $line = <$infile>) {
	# Vertex
	if($line =~ m/^Vertex\s+\d+\s+([-\d.]+)\s+([-\d.]+)\s+([-\d.]+)\s*$/) {
		print {$outfile} "v $1 $2 $3\n";
		$vertices++;
		next;
	}
	# Face
	if($line =~ m/^Face\s\d+\s(\d+)\s+(\d+)\s+(\d+)\s*$/) {
		my $f1 = $1 + 1;
		my $f2 = $2 + 1;
		my $f3 = $3 + 1;
		print {$outfile} "f $f1/$f2/$f3\n";
		$faces++;
		next;
	}
	# Blank lines/comments
	if($line =~ m/^(\s*|#.*)$/) {
		next;
	}

	# WHA?!
	print " * WARNING!  Skipping unknown line:\n".$line;
}

print "Converted $vertices vertices and $faces faces.\n";

# Clean up
close($infile);
close($outfile);
