#!/usr/bin/perl
#
# Scale an OBJ by some factor.

use strict;

# Check args
if($#ARGV != 2) {
	print "USAGE: obj_scale.pl <input.obj> <output.obj> <scale>\n";
	exit 0;
}

# Prepare files
my ($infile, $outfile);
if(!open($infile, '<', $ARGV[0])) {
	print "Can't open OBJ file ".$ARGV[0]."\n";
	exit 1;
}
if(!open($outfile, '>', $ARGV[1])) {
	print "Can't create OBJ file ".$ARGV[1]."\n";
	exit 2;
}

# Get scale
my $scale = $ARGV[2];

# Keep count of how many vertices we do
my $vertices = 0;
my $min = 999999;
my $max = -999999;
my $avg = 0;

# Loop over every line in the input file
while(my $line = <$infile>) {
	# Vertex
	if($line =~ m/^\s*v\s+([\d.-]+)\s+([\d.-]+)\s([\d.-]+)\s*$/) {
		my $v;
		foreach $v (($1, $2, $3)) {
			if($v < $min) { $min = $v; }
			if($v > $max) { $max = $v; }
			$avg += $v;
		}

		print {$outfile} "v ".($1*$scale)." ".($2*$scale).
				  " ".($3*$scale)."\n";
		$vertices++;
	} else {
		print {$outfile} $line;
	}
}

$avg /= $vertices * 3;

print "Scaled $vertices vertices by $scale.\n";
print "          min=$min, max=$max, avg=$avg\n";
print "scaled to min=".($min*$scale).", max=".($max*$scale).", avg=".($avg*$scale)."\n";

# Clean up
close($infile);
close($outfile);
