#!/usr/bin/perl
#
# Generate sine table for 16:16 fixed-point numbers

use POSIX;
use strict;

my $pi = 3.14159265358979323846;	# I don't know where perl keeps this :P

sub make_fixed {
	my $num = shift;
	my $whole = POSIX::floor($num);
	my $fract = ($num - $whole) * 65536;

	return (int($whole) << 16) + int($fract);
}

# Class header
print "// Generated by sine_gen.pl\n".
      "// Do not edit manually!\n".
      "\n".
      "public class SineTable {\n\n";

# Might as well make pi while we're here
print  "\tpublic static final int PI = ";
printf("0x%08X;\n\n", make_fixed($pi));

# Values
print "\tpublic static final int val[] = {";
for(my $i = 0; $i < 91; $i++) {
	# Newline every 5 constants
	if($i % 5 == 0) {
		print "\n\t\t";
	}
	my $fp = make_fixed(sin($pi/180 * $i));
	printf("0x%08X", $fp);
	if($i != 90) {
		print ", ";
	}
}

# Class footer
print "\n".
      "\t};\n".
      "}\n";
